﻿// ----------------------------------------------------------------------------------
// Microsoft Developer & Platform Evangelism
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
// OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// ----------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
// ----------------------------------------------------------------------------------

namespace Bowlus.Azure.ServiceManagementWrapper
{
    using System;
    using System.ServiceModel;
    using System.ServiceModel.Channels;
    using System.ServiceModel.Description;
    using System.ServiceModel.Dispatcher;

    public class ClientOutputMessageInspector : IClientMessageInspector, IEndpointBehavior
    {
        private readonly string versionHeaderContent;

        public ClientOutputMessageInspector()
            : this(Constants.DefaultVersionHeaderContent)
        {
        }

        public ClientOutputMessageInspector(string versionHeaderContent)
        {
            this.versionHeaderContent = versionHeaderContent;
        }

        public string VersionHeaderContent 
        {
            get { return this.versionHeaderContent; }
        }

        public void AfterReceiveReply(ref Message reply, object correlationState)
        { 
        }

        public object BeforeSendRequest(ref Message request, IClientChannel channel)
        {
            if (request == null)
            {
                throw new ArgumentNullException("request");
            }

            var property = (HttpRequestMessageProperty)request.Properties[HttpRequestMessageProperty.Name];
            property.Headers.Add(Constants.VersionHeaderName, this.versionHeaderContent);
            return null;
        }

        public void AddBindingParameters(ServiceEndpoint endpoint, BindingParameterCollection bindingParameters) 
        { 
        }

        public void ApplyClientBehavior(ServiceEndpoint endpoint, ClientRuntime clientRuntime)
        {
            if (clientRuntime == null)
            {
                throw new ArgumentNullException("clientRuntime");
            }

            clientRuntime.MessageInspectors.Add(this);
        }

        public void ApplyDispatchBehavior(ServiceEndpoint endpoint, EndpointDispatcher endpointDispatcher) 
        { 
        }

        public void Validate(ServiceEndpoint endpoint)
        { 
        }
    }
}