﻿namespace Bowlus.Common
{
    using System.Collections.Generic;
    using System.Linq;

    using Microsoft.WindowsAzure;
    using Microsoft.WindowsAzure.StorageClient;

    public class TenantDeploymentsStore : TableServiceContext
    {
        private const string TenantDeploymentTableName = "TenantDeployment";

        private static readonly object LockObject = new object();

        private static volatile TenantDeploymentsStore instance;

        private TenantDeploymentsStore(CloudStorageAccount account)
            : base(account.TableEndpoint.AbsoluteUri, account.Credentials)
        {
            var tableClient = new CloudTableClient(account.TableEndpoint.AbsoluteUri, account.Credentials);

            //if (
            tableClient.CreateTableIfNotExist(TenantDeploymentTableName);
            //{
            //    return;
            //}

            //var tenantDeployment = new TenantDeployment
            //    {
            //        RowKey = "initalizer",
            //        Application = "initalizer",
            //        Customizations = "initalizer",
            //        HostName = "initalizer",
            //        AppHostInstanceId = "initalizer",
            //        AppHostInstanceIp = "initalizer",
            //        Status = "initalizer",
            //        TenantId = "initalizer"
            //    };
            //this.Add(tenantDeployment);
            //this.SaveChanges();
        }

        public static TenantDeploymentsStore Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (LockObject)
                    {
                        if (instance == null)
                        {
                            return instance = new TenantDeploymentsStore(Settings.AzureStorageAccount);
                        }
                    }
                }

                return instance;
            }
        }

        public IQueryable<TenantDeployment> TenantDeployment
        {
            get
            {
                return CreateQuery<TenantDeployment>(TenantDeploymentTableName).Where(c => c.PartitionKey == "_bowlus");
            }
        }

        public static IEnumerable<TenantDeployment> GetCurrentDeployments()
        {
            // TODO: Organize deployment states
            var deployments = from tenantDeployment in Instance.TenantDeployment
                              select tenantDeployment;

            return deployments;
        }

        public static void UpdateDeploymentStore(IEnumerable<TenantDeployment> tenantDeployments)
        {
            foreach (var td in tenantDeployments)
            {
                UpdateDeploymentStore(td);
            }
        }

        public static void UpdateDeploymentStore(TenantDeployment tenantDeployment)
        {
            var tenantDeployments = Instance.TenantDeployment.ToList();

            // update tenant table
            if (tenantDeployments.Any(td => td.HostName.Equals(tenantDeployment.HostName)))
            {
                var entity = tenantDeployments.Find(td => td.HostName.Equals(tenantDeployment.HostName));
                if (entity != null)
                {
                    entity.DeepCopy(tenantDeployment);
                    Instance.Update(entity);
                }
            }
            else
            {
                Instance.Add(tenantDeployment);
            }
        }

        private void Add(TenantDeployment tenantDeployment)
        {
            this.AddObject(TenantDeploymentTableName, tenantDeployment);
            this.SaveChanges();
        }

        private void Update(TenantDeployment deployment)
        {
            this.UpdateObject(deployment);
            this.SaveChanges();
        }
    }
}
