namespace Bowlus.FrontEnd.RequestRouting.ApplicationDeployment
{
    using System;

    using Bowlus.Common;
    using Bowlus.Common.Communications;

    public class DeleteApplicationRequest : ApplicationDeploymentRequest
    {
        public DeleteApplicationRequest(TenantDeployment tenantDeployment, ApplicationHost applicationHost)
            : base(tenantDeployment)
        {
            this.FromHost = applicationHost;
        }

        public ApplicationHost FromHost { get; private set; }

        public override Guid SendDeploymentRequest()
        {
            var deploymentServiceClient = new CommunicationsClient();

            var message = this.ConstructMessage(BowlusMessageType.DeleteApp);

            deploymentServiceClient.SendMessageToRoleInstance(message, this.FromHost.DeploymentManagerEndpoint, Settings.AppHostRoleName);

            return message.MessageId;
        }

        public override bool CanUpdateConfiguration(BowlusMessage response, out Guid nextMessageId)
        {
            // If failed, result code is already in the message for delete and deploy app.
            nextMessageId = Guid.Empty;

            if (response.MessageType == BowlusMessageType.AppDeleted)
            {
                return true;
            }

            return false;
        }
    }
}