﻿namespace Bowlus.RequestRouting.Tests
{
    using System.Collections.Generic;
    using System.Linq;

    using Bowlus.Common;
    using Bowlus.FrontEnd.RequestRouting;
    using Bowlus.FrontEnd.RequestRouting.ApplicationDeployment;
    using Bowlus.TestSupport;

    using Microsoft.VisualStudio.TestTools.UnitTesting;

    public class given_a_basic_strategy : ArrangeActAssert
    {
        protected IApplicationDeploymentStrategy DeploymentStrategy;

        protected IEnumerable<ApplicationHost> ApplicationHosts;

        protected IEnumerable<TenantDeployment> Deployments;

        protected override void Arrange()
        {
            this.DeploymentStrategy = new BasicDeploymentStrategy();

            base.Arrange();
        }
    }

    [TestClass]
    public class when_all_hosts_are_present : given_a_basic_strategy
    {
        protected IEnumerable<ApplicationDeploymentRequest> DeploymentRequests;

        protected override void Act()
        {
            this.ApplicationHosts = new List<ApplicationHost>()
                {
                };

            this.Deployments = new List<TenantDeployment>()
                {
                    new TenantDeployment()
                        {
                            Application = "App1",
                            Customizations = "",
                            HostName = "APP1",
                            AppHostInstanceId = "inst_0",
                            AppHostInstanceIp = "1.1.1.1",
                            Status = "Ready",
                            TenantId = "tenant1"
                        },
                    new TenantDeployment()
                        {
                            Application = "App2",
                            Customizations = "",
                            HostName = "APP2",
                            AppHostInstanceId = "inst_1",
                            AppHostInstanceIp = "1.1.1.2",
                            Status = "Ready",
                            TenantId = "tenant2"
                        },
                    new TenantDeployment()
                        {
                            Application = "App3",
                            Customizations = "",
                            HostName = "APP3",
                            AppHostInstanceId = "inst_2",
                            AppHostInstanceIp = "1.1.1.3",
                            Status = "Ready",
                            TenantId = "tenant3"
                        },
                    new TenantDeployment()
                        {
                            Application = "App1",
                            Customizations = "",
                            HostName = "APP4",
                            AppHostInstanceId = "inst_0",
                            AppHostInstanceIp = "1.1.1.1",
                            Status = "Ready",
                            TenantId = "tenant4"
                        },
                    new TenantDeployment()
                        {
                            Application = "App2",
                            Customizations = "",
                            HostName = "APP5",
                            AppHostInstanceId = "inst_1",
                            AppHostInstanceIp = "1.1.1.2",
                            Status = "Ready",
                            TenantId = "tenant5"
                        },
                    new TenantDeployment()
                        {
                            Application = "App3",
                            Customizations = "",
                            HostName = "APP6",
                            AppHostInstanceId = "inst_2",
                            AppHostInstanceIp = "1.1.1.3",
                            Status = "Ready",
                            TenantId = "tenant6"
                        }
                };

            this.DeploymentRequests = this.DeploymentStrategy.InitializeDeployments(
                this.ApplicationHosts, this.Deployments);
            base.Act();
        }

        [TestMethod]
        public void then_applications_distributed_evenly()
        {
            Assert.AreEqual(6, this.DeploymentRequests.ToList().Count);
        }
    }
}
